
# q-expansion of newform 850.2.g.l, downloaded from the LMFDB on 05 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 850
weight = 2
poly_data = [1156, 0, 892, 0, 237, 0, 26, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0, 0, 0, 0, 0], 1], [[0, 1454, 0, 307, 0, 8, 0, -1], 272], [[7, 0, 1, 0, 0, 0, 0, 0], 1], [[0, 618, 0, 505, 0, 96, 0, 5], 272], [[-3706, 900, -2057, 406, -340, 52, -17, 2], 136], [[3706, 900, 2057, 406, 340, 52, 17, 2], 136], [[-6018, 4606, -2941, 2463, -408, 392, -17, 19], 272], [[6018, 4606, 2941, 2463, 408, 392, 17, 19], 272]]

hecke_ring_character_values = [[477, [-1, 0, 0, 0, 0, 0, 0, 0]], [751, [0, 0, 0, -1, 0, 0, 0, 0]]]
aps_data = [[1, 0, 0, 0, 0, 0, 0, 0], [0, 0, 0, 0, 0, 0, 1, 0], [0, 0, 0, 0, 0, 0, 0, 0], [0, 1, 1, 0, 0, -1, 0, 0], [1, 0, 0, 1, -2, 0, 0, 0], [0, 1, 0, 2, 0, 0, 0, 0], [-2, 0, 1, -1, 0, 0, -1, 0], [0, 1, 0, -2, -1, -1, 1, 1], [-2, 0, 0, 2, 0, 1, 0, 0], [-3, 0, 0, 3, 0, 1, 0, -1], [1, -1, -1, -1, 0, 3, 0, -1], [-2, 1, -1, -2, 1, 0, 0, 0], [1, 2, -2, 1, -2, 0, 0, 0], [4, 0, 0, 0, 0, 0, -1, 1], [0, -1, 0, 2, 0, 0, 1, 1], [-2, 0, -1, 0, -1, 1, 0, 0], [0, -1, 0, 0, 3, 3, -1, -1], [-3, -2, 2, -3, 3, 0, -1, 0], [0, 0, 0, -2, -2, -2, 2, 2], [1, -1, -1, -1, 0, 3, 0, -3], [-4, -1, 1, -4, 4, 0, 1, 0], [-2, -2, 2, -2, 3, 0, -4, 0], [-8, 0, 2, 0, 0, 0, -1, 1], [4, 0, 3, 0, -1, 1, 2, -2], [2, -1, 1, 2, -2, 0, 5, 0], [2, 0, 0, 0, 2, -2, 0, 0], [0, 0, 0, 4, -1, -1, -2, -2], [-3, -1, 1, -3, 2, 0, 0, 0], [3, 0, 0, 3, 1, 0, -1, 0], [-2, -1, -1, 2, 0, 4, 0, -1], [2, 0, -1, 0, -1, 1, 3, -3], [3, 2, 2, -3, 0, -6, 0, 2], [0, -2, 0, -4, 0, 0, -3, -3], [3, 0, 0, -3, 0, -2, 0, -4], [0, 0, 0, 0, 2, -2, 1, -1], [0, -2, 0, 0, 1, 1, 0, 0], [0, 2, 0, 2, 5, 5, 0, 0], [5, -3, -3, -5, 0, 0, 0, 0], [0, -2, 2, 0, -5, 0, 0, 0], [-4, 0, 0, -4, 3, 0, -4, 0], [0, 4, 0, 4, -2, -2, 2, 2], [0, -2, 2, 0, 3, 0, -2, 0], [2, 0, 4, 0, 0, 0, -1, 1], [7, -3, -3, -7, 0, 0, 0, -4], [2, -1, -1, -2, 0, -3, 0, 4], [5, -3, -3, -5, 0, 3, 0, -3], [5, 1, -1, 5, 0, 0, -2, 0], [-2, 0, -1, 0, -5, 5, -1, 1], [-8, 2, 2, 8, 0, -4, 0, -3], [0, 6, 0, 4, -5, -5, -1, -1], [4, 5, -5, 4, -8, 0, 3, 0], [2, 0, 6, 0, 6, -6, -1, 1], [-1, 4, 4, 1, 0, -6, 0, 4], [-10, 0, 2, 0, 2, -2, 2, -2], [-2, 0, 0, 0, 4, -4, -5, 5], [-2, 0, 3, 0, 5, -5, 1, -1], [-15, 0, 0, 15, 0, -1, 0, -1], [12, 0, -4, 0, -6, 6, -4, 4], [4, 4, -4, 4, -5, 0, -4, 0], [0, 1, 0, -2, 5, 5, 0, 0], [-8, 0, 0, 8, 0, 2, 0, -1], [0, -3, 0, 6, 10, 10, -2, -2], [0, 4, 0, 2, 2, 2, -3, -3], [-4, 0, 0, 4, 0, -9, 0, 0], [-7, 1, 1, 7, 0, -4, 0, 4], [-8, -2, -2, 8, 0, 3, 0, -2], [0, 1, 0, 8, -1, -1, -3, -3], [4, 3, -3, 4, -4, 0, 7, 0], [10, 0, 0, -10, 0, 4, 0, 3], [0, -6, 0, -6, 5, 5, 2, 2], [0, -2, 0, 4, -6, -6, 4, 4], [0, 6, 0, 14, -1, -1, 5, 5], [-6, 4, 4, 6, 0, -3, 0, 0], [0, -2, 0, 6, -1, -1, 2, 2], [9, -5, -5, -9, 0, 10, 0, 4], [-14, 0, 1, 0, 3, -3, -3, 3], [0, 2, 0, 14, -5, -5, 2, 2], [-16, -2, -2, 16, 0, 5, 0, -2], [-5, 3, -3, -5, -12, 0, 0, 0], [-8, 0, -7, 0, -5, 5, 4, -4], [-11, 1, -1, -11, -2, 0, -2, 0], [-4, 0, 4, 0, 4, -4, 3, -3], [-4, -2, 2, -4, 11, 0, -6, 0], [16, 0, 2, 0, 4, -4, 1, -1], [4, 2, 2, -4, 0, 1, 0, -2], [0, 2, 0, 18, -2, -2, 0, 0], [7, -2, 2, 7, -2, 0, -6, 0], [4, 0, 6, 0, 8, -8, -1, 1], [0, 4, 0, 18, -5, -5, -2, -2], [0, 3, 0, 14, -4, -4, -5, -5], [14, 0, 0, 0, -4, 4, -3, 3], [3, -3, -3, -3, 0, 1, 0, -5], [14, -1, -1, -14, 0, 1, 0, -4], [0, 3, 0, 6, -5, -5, 1, 1], [-3, -4, 4, -3, 2, 0, -2, 0], [8, 2, 2, -8, 0, 3, 0, 2], [-6, 0, -2, 0, -4, 4, 0, 0], [-5, -1, 1, -5, 0, 0, 0, 0], [0, 0, 0, -18, 4, 4, 2, 2], [-8, 2, 2, 8, 0, -5, 0, 10], [0, 4, -4, 0, -2, 0, 1, 0], [0, -5, 0, 2, 6, 6, 2, 2], [-6, 0, 4, 0, 4, -4, 2, -2], [0, 3, 0, -4, -1, -1, 6, 6], [-5, 2, -2, -5, 4, 0, 2, 0], [0, -10, 0, -10, 4, 4, -4, -4], [-8, 0, -2, 0, -12, 12, 2, -2], [-4, 0, -6, 0, -2, 2, 0, 0], [-4, 0, -2, 0, 2, -2, -2, 2], [-13, 3, -3, -13, -2, 0, -4, 0], [0, -1, 1, 0, -15, 0, -4, 0], [0, -3, 0, -6, -4, -4, 4, 4], [16, 1, -1, 16, 4, 0, -1, 0], [-5, 3, -3, -5, 6, 0, -6, 0], [0, 4, 0, 8, 1, 1, -2, -2], [-17, 3, 3, 17, 0, -4, 0, -6], [3, -3, 3, 3, 12, 0, -10, 0], [0, -9, 0, 6, 2, 2, 1, 1], [2, -3, -3, -2, 0, -1, 0, -2], [-10, 0, -5, 0, -5, 5, -1, 1], [0, -2, 0, -14, 1, 1, -2, -2], [6, -1, -1, -6, 0, 6, 0, -11], [-12, -3, 3, -12, 13, 0, -2, 0], [14, -4, 4, 14, 4, 0, -3, 0], [17, 2, -2, 17, 2, 0, 8, 0], [-14, 0, 6, 0, -2, 2, 0, 0], [-1, -2, -2, 1, 0, -11, 0, 9], [-11, 3, 3, 11, 0, 3, 0, -3], [0, 7, 0, -2, -2, -2, -5, -5], [-14, 0, -6, 0, -4, 4, 2, -2], [0, 5, 0, 22, -1, -1, 1, 1], [-4, -1, 1, -4, -11, 0, -8, 0], [9, -5, -5, -9, 0, 9, 0, -11], [14, 0, -11, 0, -11, 11, 0, 0], [-6, 0, -2, 0, 2, -2, -6, 6], [-10, 0, 5, 0, 5, -5, 4, -4], [10, 0, 2, 0, 8, -8, -2, 2], [-14, 8, -8, -14, -14, 0, 5, 0], [-14, 0, 2, 0, -2, 2, -4, 4], [-11, -4, 4, -11, 6, 0, 0, 0], [-1, 0, 0, 1, 0, 8, 0, 2], [1, -2, -2, -1, 0, -7, 0, -3], [2, 0, 0, -2, 0, 5, 0, -4], [19, 1, 1, -19, 0, 2, 0, 0], [2, 0, 2, 0, -2, 2, -4, 4], [-9, 3, -3, -9, -7, 0, 7, 0], [-2, -3, 3, -2, 7, 0, -10, 0], [2, -9, -9, -2, 0, 14, 0, -5], [0, 11, 0, 8, 1, 1, -1, -1], [0, -4, 0, 12, 5, 5, 4, 4], [-8, 3, 3, 8, 0, 1, 0, 0], [3, 7, 7, -3, 0, -14, 0, 10], [0, -4, 0, -20, 4, 4, -3, -3], [-18, -3, 3, -18, 1, 0, -8, 0], [-4, -4, -4, 4, 0, -10, 0, -3], [-10, -8, 8, -10, 7, 0, -6, 0], [2, 0, -10, 0, -10, 10, 5, -5], [-1, -4, 4, -1, 6, 0, 0, 0], [8, 0, 4, 0, 10, -10, 1, -1], [-3, 0, 0, -3, 5, 0, 3, 0], [4, 0, 0, 4, 14, 0, -1, 0], [0, -2, 0, -18, 10, 10, 1, 1], [0, 0, 10, 0, 2, -2, 0, 0], [0, 5, 0, -20, -3, -3, 3, 3], [6, 0, -2, 0, -4, 4, 7, -7], [6, 7, -7, 6, -15, 0, -4, 0], [27, 3, 3, -27, 0, -3, 0, -1], [-4, -1, -1, 4, 0, -1, 0, 14]]
